@@ -1,6 +1,7 @@
 
 duplicated.data.table <- function(x, ...) {
     # Relatively quick test for duplication in data.table keys using diff
+    if (!cedta()) return(NextMethod("duplicated"))
     keys <- attr(x, "sorted")
     if (is.null(keys)) stop("data table must have keys")
     res <- c(FALSE, rep(TRUE, nrow(x) - 1))
@@ -10,4 +11,9 @@ duplicated.data.table <- function(x, ...) {
     res
 }
 
-unique.data.table <- function(x, ...) x[!duplicated(x)]
+unique.data.table <- function(x, ...) {
+    if (!cedta()) return(NextMethod("unique"))
+    x[!duplicated(x)]
+}
+
+
