@@ -14,8 +14,10 @@ test.data.table = function()
             if (identical(x,y)) return()
             if (is.data.table(x) && is.data.table(y)) {
                 # drop unused levels in factors
-                for (i in which(sapply(x,is.factor))) x[[i]] = factor(x[[i]])
-                for (i in which(sapply(y,is.factor))) y[[i]] = factor(y[[i]])
+                if (length(x)) for (i in which(sapply(x,is.factor))) x[[i]] = factor(x[[i]])
+                if (length(y)) for (i in which(sapply(y,is.factor))) y[[i]] = factor(y[[i]])
+                if (length(attr(x,"row.names"))) attr(x,"row.names") = NULL  # for test 165+, i.e. x may have row names set from inheritance but y won't, consider these equal
+                if (length(attr(y,"row.names"))) attr(y,"row.names") = NULL
                 if (identical(x,y)) return()
             }
             if (is.factor(x) && is.factor(y)) {
@@ -25,12 +27,14 @@ test.data.table = function()
             }
             
         }
-        cat("Test",num,"failed\n")
+        cat("Test",num,"ran without errors but failed check:\n")
+        print(x)
+        print(y)
         assign("nfail",nfail+1,envir=parent.frame())
     }
     started.at = Sys.time()
     TESTDT = data.table(a=as.integer(c(1,3,4,4,4,4,7)), b=as.integer(c(5,5,6,6,9,9,2)), v=1:7)
-    a=b=v=z=NAME=DT=B=.SD=y=V1=V2=b_1=`a 1`=a.1=NA    # For R CMD check "no visible binding for global variable"
+    a=b=v=z=NAME=DT=B=.SD=y=V1=V2=b_1=`a 1`=a.1=d=grp=buniquename314=NA    # For R CMD check "no visible binding for global variable"
     setkey(TESTDT,a,b)
     # i.e.       a b v
     #       [1,] 1 5 1
@@ -60,13 +64,20 @@ test.data.table = function()
     test(15, TESTDT[SJ(c(-3,2,4,4,5,7,8)),v,mult="last",nomatch=0], INT(6,6,7))
     test(16, TESTDT[SJ(c(4)),v][[2]], INT(3,4,5,6))
     #test(17, suppressWarnings(TESTDT[SJ(c(4,4)),v,mult="all",incbycols=FALSE][[1]]), INT(3:6,3:6))
-    test(18, TESTDT[SJ(c(-3,2,4,8)),v,mult="all"][[2]], INT(3:6))
-    test(19, TESTDT[SJ(c(-3,2,4,8)),v,mult="all",roll=TRUE][[2]], INT(1,3:6,7))
-    test(20, TESTDT[SJ(c(-3,2,4,8)),v,mult="all",rolltolast=TRUE][[2]], INT(1,3:6))
-    test(21, TESTDT[SJ(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all"][[3]], INT(1,3:4))
-    test(22, TESTDT[SJ(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",roll=TRUE][[3]], INT(1,3:4,4,6))
-    test(23, TESTDT[SJ(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",rolltolast=TRUE][[3]], INT(1,3:4,4))
-    test(24, TESTDT[SJ(c(1,NA,4,NA,NA,4,4),c(5,5,6,6,7,9,10)),v,mult="all",roll=TRUE][[3]], INT(1,3:4,5:6,6)) # The SJ sorted the NAs in the key to the end
+    test(18, TESTDT[SJ(c(-3,2,4,8)),v,mult="all",nomatch=0][[2]], INT(3:6))
+    test(185, TESTDT[SJ(c(-3,2,4,8)),v,mult="all",nomatch=NA][[2]], INT(NA,NA,3:6,NA))
+    test(19, TESTDT[SJ(c(-3,2,4,8)),v,mult="all",roll=TRUE,nomatch=0][[2]], INT(1,3:6,7))
+    test(186, TESTDT[SJ(c(-3,2,4,8)),v,mult="all",roll=TRUE,nomatch=NA][[2]], INT(NA,1,3:6,7))
+    test(20, TESTDT[SJ(c(-3,2,4,8)),v,mult="all",rolltolast=TRUE,nomatch=0][[2]], INT(1,3:6))
+    test(187, TESTDT[SJ(c(-3,2,4,8)),v,mult="all",rolltolast=TRUE,nomatch=NA][[2]], INT(NA,1,3:6,NA))
+    test(21, TESTDT[SJ(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",nomatch=0][[3]], INT(1,3:4))
+    test(188, TESTDT[SJ(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",nomatch=NA][[3]], INT(NA,1,NA,3:4,NA,NA,NA))
+    test(22, TESTDT[SJ(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",roll=TRUE,nomatch=0][[3]], INT(1,3:4,4,6))
+    test(189, TESTDT[SJ(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",roll=TRUE,nomatch=NA][[3]], INT(NA,1,NA,3:4,4,6,NA))
+    test(23, TESTDT[SJ(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",rolltolast=TRUE,nomatch=0][[3]], INT(1,3:4,4))
+    test(190, TESTDT[SJ(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",rolltolast=TRUE,nomatch=NA][[3]], INT(NA,1,NA,3:4,4,NA,NA))
+    test(24, TESTDT[SJ(c(1,NA,4,NA,NA,4,4),c(5,5,6,6,7,9,10)),v,mult="all",roll=TRUE,nomatch=0][[3]], INT(1,3:4,5:6,6))
+    test(191, TESTDT[SJ(c(1,NA,4,NA,NA,4,4),c(5,5,6,6,7,9,10)),v,mult="all",roll=TRUE,nomatch=NA][[3]], INT(NA,NA,NA,1,3:4,5:6,6))
     # Note that the NAs get sorted to the beginning by the SJ().
 
     # i.e.       a b v      (same test matrix, repeating here for easier reading of the test cases below)
@@ -94,18 +105,26 @@ test.data.table = function()
     test(39, TESTDT[J(c(5,4,-3,8,4,7,2)),v,mult="last",nomatch=0], INT(6,6,7))
     test(40, TESTDT[J(c(4)),v,mult="all"][[2]], INT(3,4,5,6))
     test(41, TESTDT[J(c(4,4)),v,mult="all"][[2]], INT(3:6,3:6))
-    test(42, TESTDT[J(c(8,2,4,-3)),v,mult="all"][[2]], INT(3:6))
-    test(43, TESTDT[J(c(8,2,4,-3)),v,mult="all",roll=TRUE][[2]], INT(7,1,3:6))
+    test(42, TESTDT[J(c(8,2,4,-3)),v,mult="all",nomatch=0][[2]], INT(3:6))
+    test(192, TESTDT[J(c(8,2,4,-3)),v,mult="all",nomatch=NA][[2]], INT(NA,NA,3:6,NA))
+    test(43, TESTDT[J(c(8,2,4,-3)),v,mult="all",roll=TRUE,nomatch=0][[2]], INT(7,1,3:6))
+    test(193, TESTDT[J(c(8,2,4,-3)),v,mult="all",roll=TRUE,nomatch=NA][[2]], INT(7,1,3:6,NA))
     #test(44, suppressWarnings(TESTDT[J(c(8,4,2,-3)),v,mult="all",rolltolast=TRUE,incbycols=FALSE][[1]]), INT(3:6,1))
-    test(45, TESTDT[J(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all"][[3]], INT(1,3:4))
-    test(46, TESTDT[J(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",roll=TRUE][[3]], INT(1,3:4,4,6))
-    test(47, TESTDT[J(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",rolltolast=TRUE][[3]], INT(1,3:4,4))
-    test(48, TESTDT[J(c(-9,NA,4,NA,1,4,4),c(1,5,9,6,5,9,10)),v,mult="all",roll=TRUE][[3]], INT(5:6,1,5:6,6))  # this time the NAs stay where they are. Compare to test 24 above.
-
-    test(49, TESTDT[J(c(4,1,0,5,3,7,NA,4,1),c(6,5,1,10,5,2,1,6,NA)),v], INT(3,1,NA,NA,2,7,NA,3,NA))
-    test(50, TESTDT[J(c(4,1,0,5,3,7,NA,4,1),c(6,5,1,10,5,2,1,6,NA)),v,mult="last"], INT(4,1,NA,NA,2,7,NA,4,NA))
-
+    test(45, TESTDT[J(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",nomatch=0][[3]], INT(1,3:4))
+    test(194, TESTDT[J(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",nomatch=NA][[3]], INT(NA,1,NA,3:4,NA,NA,NA))
+    test(46, TESTDT[J(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",roll=TRUE,nomatch=0][[3]], INT(1,3:4,4,6))
+    test(195, TESTDT[J(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",roll=TRUE,nomatch=NA][[3]], INT(NA,1,NA,3:4,4,6,NA))
+    test(47, TESTDT[J(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",rolltolast=TRUE,nomatch=0][[3]], INT(1,3:4,4))
+    test(196, TESTDT[J(c(-9,1,4,4,4,4,8),c(1,5,5,6,7,10,3)),v,mult="all",rolltolast=TRUE,nomatch=NA][[3]], INT(NA,1,NA,3:4,4,NA,NA))
+    test(48, TESTDT[J(c(-9,NA,4,NA,1,4,4),c(1,5,9,6,5,9,10)),v,mult="all",roll=TRUE,nomatch=0][[3]], INT(5:6,1,5:6,6))  # this time the NAs stay where they are. Compare to test 24 above.
+    test(197, TESTDT[J(c(-9,NA,4,NA,1,4,4),c(1,5,9,6,5,9,10)),v,mult="all",roll=TRUE,nomatch=NA][[3]], INT(NA,NA,5:6,NA,1,5:6,6))
+    test(49, TESTDT[J(c(4,1,0,5,3,7,NA,4,1),c(6,5,1,10,5,2,1,6,NA)),v,nomatch=0], INT(3,1,2,7,3))
+    test(198, TESTDT[J(c(4,1,0,5,3,7,NA,4,1),c(6,5,1,10,5,2,1,6,NA)),v,nomatch=NA], INT(3,1,NA,NA,2,7,NA,3,NA))
+    test(50, TESTDT[J(c(4,1,0,5,3,7,NA,4,1),c(6,5,1,10,5,2,1,6,NA)),v,mult="last",nomatch=0], INT(4,1,2,7,4))
+    test(199, TESTDT[J(c(4,1,0,5,3,7,NA,4,1),c(6,5,1,10,5,2,1,6,NA)),v,mult="last",nomatch=NA], INT(4,1,NA,NA,2,7,NA,4,NA))
+    
     TESTDT$a = factor(letters[TESTDT$a])
+    setkey(TESTDT,a,b)
     # i.e.       a b v
     #       [1,] a 5 1
     #       [2,] c 5 2
@@ -114,10 +133,13 @@ test.data.table = function()
     #       [5,] d 9 5
     #       [6,] d 9 6
     #       [7,] g 2 7
-    test(51, TESTDT[SJ(c("d","d","e","g"),c(6,7,1,2)),v,mult="all",roll=TRUE][[3]], INT(3:4,4,7))  # will test sortedmatch for strings in the level match
-    test(52, TESTDT[J(c("g","d","e","d"),c(6,6,1,2)),v,mult="all",roll=TRUE][[3]], INT(7,3:4))  # also will test sortedmatch for strings in the level match
+    test(51, TESTDT[SJ(c("d","d","e","g"),c(6,7,1,2)),v,mult="all",roll=TRUE,nomatch=0][[3]], INT(3:4,4,7))  # will test sortedmatch for strings in the level match
+    test(200, TESTDT[SJ(c("d","d","e","g"),c(6,7,1,2)),v,mult="all",roll=TRUE,nomatch=NA][[3]], INT(3:4,4,NA,7))
+    test(52, TESTDT[J(c("g","d","e","d"),c(6,6,1,2)),v,mult="all",roll=TRUE,nomatch=0][[3]], INT(7,3:4))  # also will test sortedmatch for strings in the level match
+    test(201, TESTDT[J(c("g","d","e","d"),c(6,6,1,2)),v,mult="all",roll=TRUE,nomatch=NA][[3]], INT(7,3:4,NA,NA))
 
     TESTDT$b = factor(letters[TESTDT$b])
+    setkey(TESTDT,a,b)
     # i.e.
     #         a b v
     #    [1,] a e 1
@@ -159,14 +181,17 @@ test.data.table = function()
     test(68, "TESTDT" %in% tables(silent=TRUE)[,NAME])  # NAME is returned as a column in which we look for the string 
     test(69, "TESTDT" %in% tables(silent=TRUE)[,as.character(NAME)]) # an old test (from when NAME was factor) but no harm in keeping it
 
-    a = "d"     # Variable Twister.  a in this scope has same name as a inside DT scope.
-    test(70, TESTDT[J(a),DT(v)], data.table(a="d",v=3:6)) # J(a) means use a we just set above,  not a inside the DT which would result in a self join of the whole table. Would only occur if there is a variable name conflict as deliberately created here.
-    test(71, TESTDT[SJ(a),DT(v)], data.table(a="d",v=3:6,key="a"))
-    test(72, TESTDT[CJ(a),DT(v)], data.table(a="d",v=3:6,key="a"))
+    a = "d"
+    # Variable Twister.  a in this scope has same name as a inside DT scope.
+    # Aug 2010 : As a result of bug 1005, and consistency with 'j' and 'by' we now allow self joins (test 183) in 'i'.
+    test(70, TESTDT[eval(J(a)),v], data.table(a="d",v=3:6))   # the eval() enabled you to use the 'a' in the calling scope, not 'a' in the TESTDT
+    test(71, TESTDT[eval(SJ(a)),v], data.table(a="d",v=3:6,key="a"))
+    test(72, TESTDT[eval(CJ(a)),v], data.table(a="d",v=3:6,key="a"))
 
     test(73, TESTDT[,v], 1:7)
-    test(74, TESTDT[,3], data.table(v=INT(1:7)))
-    test(75, TESTDT[,"v"], data.table(v=INT(1:7)))
+    test(74, TESTDT[,3], 3)
+    test(74.5, TESTDT[,3L], 3L)
+    test(75, TESTDT[,"v"], "v")
     test(76, TESTDT[,2:3], 2:3)  # See ?[.data.table that with=FALSE is required for the likely intended result
     test(77, TESTDT[,2:3,with=FALSE], data.table(b=c("e","e","f","f","i","i","b"),v=1:7))
     test(78, TESTDT[,c("b","v"),with=FALSE], data.table(b=c("e","e","f","f","i","i","b"),v=1:7))
@@ -207,9 +232,9 @@ test.data.table = function()
     (dt <- data.table(A = rep(1:3, each=4), B = rep(1:4, each=3), C = rep(1:2, 6), key = "A,B"))
     test(99, unique(dt), data.table(dt[c(1L, 4L, 5L, 7L, 9L, 10L)], key="A,B"))
 
-    # test [<- for column assignment (row assignment is still messed up)
+    # test [<- for column assignment 
     dt1 <- dt2 <- dt
-    test(100, {dt1["A"] <- 3; dt1}, {dt2$A <- 3; dt2})
+    test(100, {dt1[,"A"] <- 3; dt1}, {dt2$A <- 3; dt2})
 
     # test transform and within
     test(101, within(dt, {D <- B^2}), transform(dt, D = B^2))
@@ -235,6 +260,7 @@ test.data.table = function()
 
     # test [<- assignment:
     dt2[is.na(dt2)] <- 1L
+    setkey(dt2, A, B)
     test(113, dt, dt2)
     dt2[, c("A", "B")] <- dt1[, c("A", "B"), with = FALSE]
     test(114, dt1, dt2)
@@ -276,9 +302,9 @@ test.data.table = function()
     test(125, inherits(t,"try-error"))
     test(126, length(grep("The data.table has no key", t)))
 
-    TESTDT = data.table(a=3L,v=2,key="a")
+    TESTDT = data.table(a=3L,v=2,key="a")  # testing 1-row table
     test(127, TESTDT[J(3)], TESTDT)
-    test(128, TESTDT[J(4)], TESTDT[NA])
+    test(128, TESTDT[J(4)], TESTDT[NA])   # see tests 206-207 too re the [NA]
     test(129, TESTDT[J(4),roll=TRUE], TESTDT)
     test(130, TESTDT[J(4),rolltolast=TRUE], TESTDT[NA])
     test(131, TESTDT[J(-4),roll=TRUE], TESTDT[NA])
@@ -350,7 +376,169 @@ test.data.table = function()
     test(156, is.unsorted(levels(a$y)), TRUE)   # non-key columns are ok to have unsorted levels
     test(157, a[,sum(z),by=x][1,paste(x,V1)], "a 3")
     test(158, a[,sum(z),by=y], before)
+
+    # tests of by expression variables
+    DT = data.table( a=1:5, b=11:50, d=c("A","B","C","D"), f=1:5, grp=1:5 )
+    f = quote( list(d) )
+    test(159, DT[,mean(b),by=eval(f)], DT[,mean(b),by=list(d)])  # column f doesn't get in the way of expression f
+    foo = function( grp ) {
+       DT[,mean(b),by=eval(grp)]
+    }
+    test(160, foo(quote(list(d))), DT[,mean(b),by=list(d)])
+    test(161, foo(quote(list(d,a))), DT[,mean(b),by=list(d,a)])
+    test(162, foo(quote(list(f))), DT[,mean(b),by=list(f)])
+    test(163, foo(quote(list(grp))), DT[,mean(b),by=list(grp)])  # grp local variable in foo doesn't conflict with column grp
+    test(164, foo(f), DT[,mean(b),by=d])
     
+    # checks that data.table inherits methods from data.frame in base ok
+    test(165, subset(DT,a>2), DT[a>2])
+    test(166, suppressWarnings(split(DT,DT$grp)[[2]]), DT[grp==2])
+    if ("package:ggplot2" %in% search()) {
+        test(167,print(ggplot(DT,aes(b,f))+geom_point()),NULL)  # how to programmatically test it not only doesn't error but correct output, binary diff to pre-prepared pdf ?
+        test(168,DT[,print(ggplot(.SD,aes(b,f))+geom_point()),by=list(grp%%2L)],data.table(grp=integer()))  # %%2 because there are 5 groups in DT data at this stage, just need 2 to test
+        #try(graphics.off(),silent=TRUE) # R CMD check doesn't like graphics it seems, even when inside try()
+    } else {
+        cat("Tests 167 and 168 not run. If required call library(ggplot2) first.\n")
+        # ggplot takes a long time e.g. increases runtime of test.data.table from under 1 second to over 10 seconds. So we don't include these by default.
+        # From examples, the library(ggplot2) is done first, so that 'R CMD check' does include tests 167 and 168 
+    }
+    # test of . in formula, using inheritance
+    # never mind that grp column is included (NA coef), thats another todo for later.
+    DT = data.table(y=1:100,x=101:200,y=201:300,grp=1:5)
+    test(169,DT[,as.list(lm(y~0+.,.SD)$coef),by=grp][2,x]-2<1e-10, TRUE)
+    
+    DT <- data.table( a=1:4, d=c("A","B","C","D") )
+    g <- quote( list( d ) )
+    test(170, DT[,list(d)], DT[,eval(g)])
+    
+    DT = data.table(A=c(25L,85L,25L,25L,85L), B=c("a","a","b","c","c"), C=c(2,65,9,82,823))
+    test(171, DT[ , data.table( A, C )[ A==25, C ] + data.table( A, C )[ A==85, C ], by=B ], data.table(B=c("a","c"),V1=c(67,905)))
+    
+    test(172, DT[ , list(3,data.table( A, C )[ A==25, C ] + data.table( A, C )[ A==85, C ]), by=B ], data.table(B=c("a","b","c"),V1=3,V2=c(67,NA,905)))
+    
+    # Test growing result in memory. Usually the guess is good though.
+    # This example returns no rows for first group so guess for up-front allocate needs a reallocate
+    DT = data.table(A=c(1L,1L,2L,2L,3L,3L), B=1:6)
+    test(173, DT[,B[B>3],by=A][,V1], c(4L,5L,6L))
+
+    # Example taken from Harish post to datatable-help on 11 July
+    DT <- data.table(
+         A=c("a","a","b","b","d","c","a","d"),
+         B=c("x1","x2","x2","x1","x2","x1","x1","x2"),
+         C=c(5,2,3,4,9,5,1,9)
+         )
+    test(174, DT[,C[C-min(C)<3],by=list(A,B)][,V1], c(1,2,4,3,5,9,9))
+    test(175, DT[,C[C-min(C)<5],by=list(A,B)][,V1], c(5,1,2,4,3,5,9,9))
+
+    # Tests of data.table sub-assignments: $<-.data.table & [<-.data.table
+    DT <- data.table(a = c("A", "Z"), b = 1:10, key = "a")
+    DT[J("A"),2] <- 100
+    DT[J("A"),"b"] <- 1:5
+    DT[1:3,"b"] <- 33
+    test(176, DT,  data.table(a = rep(c("A", "Z"), each = 5),
+                              b = c(rep(33, 3), 4:5, seq(2, 10, by = 2)),
+                              key = "a"))
+    DT[J("A"),"a"] <- "Z"
+    test(177, key(DT), NULL )
+
+    DT <- data.table(a = c("A", "Z"), b = 1:10, key = "a")
+    DT$b[1:5] <- 1:5
+    DT$b[1:3] <- 33
+    test(178, DT,  data.table(a = rep(c("A", "Z"), each = 5),
+                              b = c(rep(33, 3), 4:5, seq(2, 10, by = 2)),
+                              key = "a"))
+    DT$a <- 10:1
+    test(179, key(DT), NULL )
+    
+    # Test logical in a key
+    DT = data.table(a=rep(1:3,each=2),b=c(TRUE,FALSE),v=1:6)
+    setkey(DT,a,b)
+    test(180, DT[J(2,FALSE),v], 4L)
+    test(181, DT[,sum(v),by=b][,V1], c(12L,9L))
+
+    # Test fix for bug 1026 reported by Harish V
+    rm(buniquename314)
+    colnames(DT)[2] = "buniquename314"  # this test needed a unique var name to generate error 'object 'b' not found'. Otherwise it finds 'b' in local scope.   
+    boo = function( data, fcn ) {
+        q = substitute( fcn )
+        xx = data[,eval(q),by=a]
+        yy = data[,eval(substitute(fcn)),by=a]
+        identical(xx,yy)
+    }
+    test(182, boo( DT, sum(buniquename314) ), TRUE)
+    
+    # Test bug 1005 reported by Branson Owen
+    DT = data.table(A = c("o", "x"), B = 1:10, key = "A")
+    test(183, DT[J(unique(A)), B], 1:2)
+    
+    # Test bug 709
+    xx = data.table(a=1:5,b=6:10)
+    test(184, xx[a>6,sum(b),by=a], 0L)   # aside: consistent with sum(NULL)==0
+    
+    # Tests of bug 1015 highlight by Harish
+    # See thread "'by without by' now heeds nomatch=NA"
+    # Tests 185-201 were added in above next to originals
+    x <- data.table(a=c("a","b","d","e"),b=c("A","A","B","B"),d=c(1,2,3,4), key="a,b")
+    y <- data.table(g=c("a","b","c","d"),h=c("A","A","A","A"))
+    test(202, x[y], x[y,mult="all"])
+    test(203, x[y,d], c(1,2,NA,NA))
+    test(204, x[y,list(d)][[1]], c(1,2,NA,NA))
+    test(205, x[y,list(d),mult="all"][,d], c(1,2,NA,NA))
+    
+    # Test [NA] returns one NA row. NA is type *logical* so prior to
+    # change in v1.5, NA would get silently recycled and the whole table would
+    # be returned all NA (rarely useful and often confusing, but consistent
+    # with data.frame).
+    TESTDT = data.table(a=1:3,v=1:3,key="a")
+    test(206, TESTDT[NA], data.table(a=NA_integer_,v=NA_integer_,key="a"))
+    key(TESTDT) = NULL
+    test(207, TESTDT[NA], data.table(a=NA_integer_,v=NA_integer_))
+    
+    # With inheritance, NROW and NCOL in base work nicely. No need for them in data.table.
+    test(208, NROW(TESTDT), 3L)
+    test(209, nrow(TESTDT), 3L)
+    test(210, NCOL(TESTDT), 2L)
+    test(211, ncol(TESTDT), 2L)
+    
+    # Test infinite recursion error is trapped when a pre-1.5 data.table
+    # is used with 1.5 (bug #1008)
+    DT = data.table(a=1:6,key="a")
+    test(212, DT[J(3),a], 3L) # correct class c("data.table","data.frame")
+    class(DT) = "data.table"  # incorrect class
+    tt = try(DT[J(3),a], silent=TRUE)
+    test(213, inherits(tt,"try-error"))
+    test(214, length(grep("data.table inherits from data.frame", tt)))
+    
+    # setkey now auto coerces double and character for convenience, and
+    # to solve bug #953
+    DF = data.frame(a=LETTERS[1:10], b=1:10, stringsAsFactors=FALSE)
+    DT = data.table(DF)
+    key(DT) = 'a'   # used to complain about character
+    test(215, DT["C",b], 3L)
+    DT = data.table(DF,key="a")
+    test(216, DT["C",b], 3L)
+    DT = data.table(a=c(1,2,3),v=1:3,key="a")
+    test(217, DT[J(2),v], 2L)
+    DT = data.table(a=c(1,2.1,3),v=1:3)
+    tt = try(setkey(DT,a), silent=TRUE)
+    test(218, inherits(tt,"try-error"))
+    test(219, length(grep("losing information", tt)))
+    
+    # tests of quote()-ed expressions in i. Bug #1058
+    DT = data.table(a=1:5,b=6:10,key="a")
+    q = quote(a>3)
+    test(220, DT[eval(q),b], 9:10)
+    test(221, DT[eval(parse(text="a>4")),b], 10L)
+    test(222, DT[eval(parse(text="J(2)")),b], 7L)
+    
+    # lists in calling scope should be ok as single names passed to by, bug #1060
+    DT = data.table(a=1:2,b=rnorm(10))
+    byfact = DT[,a]   # vector, ok before fix but check anyway
+    test(223, DT[,mean(b),by=byfact], DT[,mean(b),by=list(byfact)])
+    byfact = DT[,list(a)]  # this caused next line to fail before fix
+    test(224, DT[,mean(b),by=byfact], DT[,mean(b),by=as.list(byfact)])
+    test(225, DT[,mean(b),by=byfact], DT[,mean(b),by={byfact}])
+
 
     ##########################
     if (nfail > 0) {
